<?php
/*
a(tom)Blog is a atom powered weblog system.
Copyright (C) 2004  Yoan BLANC

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

include_once(dirname(__FILE__)."/class/xslt.class.php");
include_once(dirname(__FILE__)."/inc/pwd.inc.php");
include_once(dirname(__FILE__)."/inc/fct.inc.php");

/**
 * An data server, atom & xslt powered
 *
 * todo (because it's only a version 0.1) :
 *  - RSS 1.0 feed
 *  - RSS 2.0 feed
 *  - atom archives
 *  - search
 *
 * @author	Yoan Blanc <greut [at] altern.org>
 * @version 0.1 040625
 */
class atomServer
{
	/// tranformations tool
	var $xslt;
	/// path to this class
	var $data;
	/// url of the server
	var $urlbase;
	/// atom:id base
	var $atomid;
	
	/// xml files
	var $xml;
	/// xsl files for php, xml, xhtml, rss output
	var $xsl;
	
	/// initial files for entry and feed
	var $init;
	
	/**
	 * Constructeur du Serveur Atom
	 *
	 * @param $data	relative path to this file
	 * @param $urlbase url of the website
	 */
	function atomServer($data, $urlbase)
	{
		$this->data = $data;
		$this->urlbase = $urlbase;
		$this->atomid = "atom.dosimple.ch";
		$this->xslt = new Xslt();
		$this->xslt->setEngine("domxml");
		
		// XML
		$this->xml["base"] = '/data/';
        $this->xml["users"] = $this->xml["base"].'users.xml';
		
		// XSLT
		// - PHP
		$this->xsl["php-fetch-user"] = '/xsl/php-fetch-user.xsl';
		$this->xsl["php-users"] = '/xsl/php-users.xsl';
		$this->xsl["php-feed"] = '/xsl/php-feed.xsl';
                $this->xsl["php-entry"] = '/xsl/php-entry.xsl';
		// - XML
		$this->xsl["xml-edit-user"] = '/xsl/xml-edit-user.xsl';
		$this->xsl["xml-del-user"] = '/xsl/xml-del-user.xsl';
		$this->xsl["xml-add-user"] = '/xsl/xml-add-user.xsl';
		$this->xsl["xml-entry"] = '/xsl/xml-entry.xsl';
		$this->xsl["xml-add-entry"] = '/xsl/xml-add-entry.xsl';
		$this->xsl["xml-del-entry"] = '/xsl/xml-del-entry.xsl';
		$this->xsl["xml-feed"] = '/xsl/xml-feed.xsl';
		
		// - XHTML
		$this->xsl["xhtml-feed"] = '/xsl/xhtml-feed.xsl';
		$this->xsl["xhtml-entry"] = '/xsl/xhtml-entry.xsl';
		$this->xsl["xhtml-archives"] = '/xsl/xhtml-archives.xsl';
		
		// - RSS
		$this->xsl["rss091-feed"] = '/xsl/rss091-feed.xsl';
		
		// - API
		$this->xsl["api-add-entry"] = '/xsl/api-add-entry.xsl';
		$this->xsl["api-edit-entry"] = '/xsl/api-edit-entry.xsl';
		
		// init
		$this->init['feed'] = '/xml/feed_model.xml';
		$this->init['entry'] = '/xml/entry_model.xml';
		$this->init['introspection'] = '/xml/introspection_model.xml';
	}
	
	/**
	 * The user's informations.
	 * If $login is empty you've got all the users
	 *
	 * @param $login	user's login
	 * @return user's information
	 */
	function getUserData($login='')
	{		
		if(!empty($login))
		{		
			$varname = 'userdata';
			
			eval($this->__getXsltResult($this->xml["users"],
				$this->xsl["php-fetch-user"],
				array(
					'login'=>$login,
					'varname'=>$varname))
			);
			
			return $$varname;
		}
		else
		{
			return getAllUsersData();
		}
	}
	
	/**
	 * The users' informations
	 *
	 * @return users' informations
	 */
	function getAllUsersData()
	{
		$varname = 'users';
		
		eval($this->__getXsltResult(
			$this->xml["users"],
			$this->xsl["php-users"],
			array('varname'=>$varname))
		);
		
		return $$varname;
	}
	
	/**
	 * Update an user
	 *
	 * @param $data	user's information (array)
	 */
	function setUserData($data)
	{
		$user = $this->__getXsltResult(
			$this->xml["users"],
			$this->xsl["xml-edit-user"],
			$data);
		
		// sauvegarde des modifications
		file_put_contents($this->data.$this->xml["users"],$user);
	}
	
	/**
	 * Add an user to users' list
	 *
	 * @param $data	user's information (array)
	 */
	function addUser($data)
	{
		if($this->getUserData($data['login']) == '')
		{
			$user = $this->__getXsltResult(
				$this->xml["users"],
				$this->xsl["xml-add-user"],
				$data);
			
			// sauvegarde des modifications
			file_put_contents($this->data.$this->xml["users"],$user);
			mkdir($this->data.'/data/'.$data['login']);
			$emptyfeed = file_get_contents($this->data.$this->init['feed']);
			return file_put_contents($this->data.'/data/'.$data['login'].'/feed.xml',$emptyfeed,FILE_APPEND);
		}
		else
		{
			return false;
		}
	}
	
	/**
	 * Remove an user for the list
	 *
	 * @param $login user's login
	 */
	function delUser($login)
	{
		$user = $this->__getXsltResult(
			$this->xml["users"],
			$this->xsl["xml-del-user"],
			array('login'=>$login));
		
		// sauvegarde des modifications
		file_put_contents($this->data.$this->xml["users"],$user);
		deldir($this->data.'/data/'.$login);
	}
	
	/**
	 * Return an entry
	 *
	 * @param $login	user's login
	 * @param $entry_id	entry's id
	 * @return atom entry
	 */
	function getEntry($login,$entry_id)
	{
		$entry = $this->__getXsltResult(
			$this->xml['users'],
			$this->xsl['xml-entry'],
			array(
					'login'=>$login,
					'id'=>$entry_id
			));
		return $entry;
	}
	
	/**
	 * Return an entry
	 *
	 * @param $login	user's login
	 * @param $entry_id	entry's id
	 * @return xhtml entry
	 */
	function getXhtmlEntry($login,$entry_id)
	{
		$feed = $this->__getXsltResult(
			$this->xml['users'],
			$this->xsl['xhtml-entry'],
			array(
				'login'=>$login,
				'id'=>$entry_id
			));
			
		// anti-SPAM
		$feed = preg_replace_callback( '#href="mailto:(.*?)"#si','antispam',$feed);
		$feed = str_replace( '@', '&#64;',$feed);
		
		return $feed;
	}
	
	/**
	 * Return an entry
	 *
	 * @param $login	user's login
	 * @param $entry_id	entry's id
	 * @return php array entry
	 */
	function getEntryData($login,$entry_id)
	{
		$varname = 'entryData';
		
		eval($this->__getXsltResult($this->xml['base'].$login.'/'.$entry_id,
			$this->xsl["php-entry"],
			array(
				'varname'=>$varname
			)));
		
		$file = file_get_contents($this->data.$this->xml['base'].$login.'/'.$entry_id);
		
		$elem = array('content','summary','title');
		foreach($elem as $tag)
		{
			${$varname}[$tag] = ((preg_match('#<'.$tag.'(.*?)>(.*?)</'.$tag.'>#si',$file,$match))? $match[2] : '');
		}
		
		return $$varname; 
	}
	
	/**
	 * Add an entry
	 *
	 * @param $login	user's login
	 * @param $entry_id	entry's id
	 * @param $data		entry data
	 * @return true if ok
	 */
	function addEntryData($login, $data)
	{
		//-- time
		$now = time();
		
		$entry_id = date("Y/m/d_His",$now).'.xml';
		$data['issued'] = date("Y-m-d",$now).'T'.date("H:i:s",$now).'+01:00';
		$data['modified'] = $data['issued'];
		$data['id'] = 'tag:'.$this->atomid.','.date('Y-m-d',$now).':/'.$data['user'].'/'.$entry_id;
		
		//-- entry
		if(!($file = file_get_contents($this->data.$this->init['entry'])))
		{
			return false;
		}
		
		foreach($data as $key => $value)
		{
			$file = preg_replace('#<'.$key.'(.*?)>(.*?)</'.$key.'>#si','<'.$key.'\1>'.$value.'</'.$key.'>',$file);
		}
		
		return $this->__addEntry($login, $entry_id, $file);
	}
	
	/**
	 * Update an entry
	 *
	 * @param $login 	user's login
	 * @param $entry_id	entry's id
	 * @param $data		entry's data
	 * @return true if ok
	 */
	function setEntryData($login, $entry_id, $data)
	{
		$now = time();
		$data['modified'] = date("Y-m-d",$now).'T'.date("H:i:s",$now).'+01:00';
		
		if(!($file = file_get_contents($this->data.$this->xml['base'].$login.'/'.$entry_id)))
		{
			return false;
		}
		
		foreach($data as $key => $value)
		{
			$file = preg_replace('#<'.$key.'(.*?)>(.*?)</'.$key.'>#si','<'.$key.'\1>'.$value.'</'.$key.'>',$file);
		}
		
		return $this->__setEntry($login, $entry_id, $file);
	}
	
	/**
	 * Update an entry.
	 *
	 * @param $login 	user's login
	 * @param $entry_id	entry's id
	 * @param $xml		updated entry (atom format)
	 * @return true if ok
	 */
	function setEntry($login, $entry_id, $xml)
	{
		//-- info
		$info = $this->getEntryData($login,$entry_id);
				
		//-- time
		$now = time();
		
		$atomid = $info['id'];
		$issued = $info['issued'];
		$modified = date("Y-m-d",$now).'T'.date("H:i:s",$now).'+01:00';
		
		$tmpfile = $this->xml['base'].$login.'/'.$now;
		file_put_contents($this->data.$tmpfile,$xml);
		
		$entry = $this->__getXsltResult(
			$tmpfile,
			$this->xsl["api-edit-entry"],
			array(
				'atomid'=>$atomid,
				'issued'=>$issued,
				'modified'=>$modified
			));
			
		@unlink($this->data.$tmpfile);
		
		return $this->__setEntry($login, $entry_id, $entry);
	}
	
	/** [private method]
	 * Update an entry.
	 *
	 * @param $login 	user's login
	 * @param $entry_id	entry's id
	 * @param $xml		updated entry (local atom format)
	 * @return true if ok
	 */
	function __setEntry($login, $entry_id, $xml)
	{
		return file_put_contents($this->data.$this->xml['base'].$login.'/'.$entry_id,$xml);
	}
	
	/**
	 * Add an entry.
	 *
	 * @param $login 	user's login
	 * @param $xml		new entry (atom format)
	 * @return true if ok
	 */
	function addEntry($login,  $xml)
	{
		//-- time
		$now = time();
		
		$entry_id = date("Y/m/d_His",$now).'.xml';
		
		$issued = date("Y-m-d",$now).'T'.date("H:i:s",$now).'+01:00';
		$modified = $issued;
		$atomid = 'tag:'.$this->atomid.','.date('Y-m-d',$now).':/'.$login.'/'.$entry_id;
		
		$tmpfile = $this->xml['base'].$login.'/'.$now;
		file_put_contents($this->data.$tmpfile,$xml);
		
		if(!$entry = $this->__getXsltResult(
			$tmpfile,
			$this->xsl["api-add-entry"],
			array(
				'atomid'=>$atomid,
				'issued'=>$issued,
				'modified'=>$modified
			)))
		{
			@unlink($this->data.$tmpfile);
			return false;
		}
		@unlink($this->data.$tmpfile);
		
		return $this->__addEntry($login, $entry_id, $entry);
	}
	
	/**
	 * Write on the disk the new entry
	 *
	 * @param $login 	user's login
	 * @param $entry_id	entry's id
	 * @param $xml		new entry (local atom format)
	 * @return true if ok
	 */
	function __addEntry($login, $entry_id, $xml)
	{
		//-- filesystem
		$url = explode('/',$entry_id);
		$base = $this->data.$this->xml['base'].$login.'/'.$url[0];
		// 2004
		if(!file_exists($base))
		{
			@mkdir($base,0755);
		}
		
		$base .= '/'.$url[1];
		// 06
		if(!file_exists($base))
		{
			@mkdir($base,0755);
		}
		
		// feed
		if($this->__addEntryToFeed($login, $entry_id))
		{
			return file_put_contents($this->data.$this->xml['base'].$login.'/'.$entry_id,$xml);
		}
		else
		{
			return false;
		}
	}
	
	/** [private method]
	 * Add an entry to the user's feed
	 *
	 * @param $login	user's login
	 * @param $entry_id	entry's id
	 * @return true if ok
	 */
	function __addEntrytoFeed($login, $entry_id)
	{
		$feed = $this->__getXsltResult(
			$this->xml['base'].$login.'/feed.xml',
			$this->xsl['xml-add-entry'],
			array(
				'id'=>$entry_id,
				'position'=>0
			));
		return file_put_contents($this->data.$this->xml['base'].$login.'/feed.xml',$feed);
	}
	
	/**
	 * Delete an entry
	 *
	 * @param $login user's login
	 * @param $entry_id entry's id
	 * @return true if ok
	 */
	function delEntry($login, $entry_id)
	{
		$url = explode('/',$entry_id);
		@unlink($this->data.$this->xml['base'].$login.'/'.$entry_id);
		//-- filesystem
		$base = $this->data.$this->xml['base'].$login.'/'.$url[0];
		@rmdir($base,0755);
		$base .= '/'.$url[1];
		@rmdir($base,0755);
		
		return $this->__delEntryToFeed($login, $entry_id);
	}
	
	/** [private method]
	 * Remove an entry from the user's feed
	 *
	 * @param $login user's login
	 * @param $entry_id entry's id
	 * @return true if ok
	 */
	function __delEntryToFeed($login, $entry_id)
	{
		$feed = $this->__getXsltResult(
			$this->xml['base'].$login.'/feed.xml',
			$this->xsl['xml-del-entry'],
			array(
				'id'=>$entry_id
			));
		return file_put_contents($this->data.$this->xml['base'].$login.'/feed.xml',$feed);
	}
	
	/**
	 * Return the user's feed
	 *
	 * @param $login	user's login
	 * @param $quantity quantity of entries
	 * @return atom feed
	 */
	function getFeed($login, $quantity=10)
	{
		return $this->__getXsltResult(
			$this->xml['users'],
			$this->xsl['xml-feed'],
			array(
				'login'=>$login,
				'quantity'=>$quantity
			));
	}
	
	/**
	 * Return the user's feed
	 *
	 * @param $login	user's login
	 * @param $quantity quantity of entries
	 * @return xhtml feed
	 */
	function getXhtmlFeed($login, $quantity=10)
	{
		$feed = $this->__getXsltResult(
			$this->xml['users'],
			$this->xsl['xhtml-feed'],
			array(
				'login'=>$login,
				'quantity'=>$quantity
			));
			
		// anti-SPAM
		$feed = preg_replace_callback( '#href="mailto:(.*?)"#si','antispam',$feed);
		$feed = str_replace( '@', '&#64;',$feed);
		
		return $feed;
	}
	
	/**
	 * Return the user's feed
	 *
	 * @param $login	user's login
	 * @param $quantity quantity of entries
	 * @return rss 0.91 feed
	 */
	function getRss091Feed($login, $quantity=10)
	{
		return $this->__getXsltResult(
			$this->xml['users'],
			$this->xsl['rss091-feed'],
			array(
				'login'=>$login,
				'quantity'=>$quantity
			));
	}
	
	/**
	 * Return the user's feed
	 *
	 * @param $login	user's login
	 * @param $start 	starting entry
	 * @param $quantity quantity of entries
	 * @return php array feed
	 */
	function getFeedData($login,$start=0,$quantity=10)
	{
		$varname = 'userFeed';
		
		eval($this->__getXsltResult($this->xml['users'],
                        $this->xsl["php-feed"],
                        array(
                                'login'=>$login,
                                'varname'=>$varname,
                                'quantity'=>$quantity,
                                'offset'=>$start
                        )));
		
		return $$varname;
	}
	
	/**
	 * Get archives
	 *
	 * @param $login 	user's login
	 * @param $date		date of archives ({YYYY}, {YYYY/MM} or {YYYY/MM/DD})
	 * @return xhtml archives
	 */
	function getXhtmlArchives($login,$date)
	{
		$feed = $this->__getXsltResult(
			$this->xml['users'],
			$this->xsl['xhtml-archives'],
			array(
				'login'=>$login,
				'archive'=>$date
			));
		
		return $feed;
	}
	
	/**
	 * Get instrospection file
	 *
	 * @param $login user's login
	 * @return introspection file
	 */
	function getIntrospection($login)
	{
		$file = file_get_contents($this->data.$this->init['introspection']);
		return str_replace('$user',$login,$file);
	}
	
	/** [private method]
	 * Return the result of a XSLT transformation
	 *
	 * @param $xml		xml file
	 * @param $xsl		stylesheet file
	 * @param $param	transformation parameters
	 * @return result of the transformation
	 */
	function __getXsltResult($xml,$xsl,$param=NULL)
	{
		$this->__XsltTransform($xml,$xsl,$param);
		$data = $this->xslt->result();
		
		return $data;
	}
	
	/** [private method]
	 * Apply a transformation
	 *
	 * @param $xml		XML file
	 * @param $xsl		stylesheet file
	 * @param $param	transformation parameters
	 */
	function __XsltTransform($xml,$xsl,$param=NULL)
	{
		$this->xslt->setXml($this->data.$xml);
		$this->xslt->setXsl($this->data.$xsl);
		$this->xslt->setParam($param);
		
		$this->xslt->transform();
	}
}
?>